---
author: Steve Andrews
---

!!! todo 
    Author name and date.

# About `libsmoldyn`

Also see [Python Bindings](PythonBindings.md).

`libsmoldyn` is a C and C++ interface to the Smoldyn simulator. `libsmoldyn` is
complementary to the stand-alone Smoldyn program in that it is a little more
difficult to use, but it provides much more flexibility. In addition,
`libsmoldyn` provides: 

1. an application programming interface that will be relatively stable, even as
   Smoldyn is updated and improved, 
2. function names that are relatively sensible and that shouldn’t collide with
   other function names in other software, and 
3. reasonably thorough error checking in every function which helps ensure that
   the user is using the function in a sensible way and in a way that won’t
   crash Smoldyn.

I initially planned to use SWIG to provide `libsmoldyn` access from several
languages but have not pursued this recently. Carlos Lopez’s group was adding
`libsmoldyn` to PySB for a while, to give it Python accessibility, although that
project is not ongoing currently.

`libsmoldyn` only barely supports graphics at present due to constraints imposed
by the glut code library. This will be improved in a future version (by
changing to the freeglut library, which doesn’t insist on controlling the main
event loop, as the glut library does).

## Compiling and linking

### Compiling

#### Header files

To enable a C or C++ program to call `libsmoldyn`, it has to include the
`libsmoldyn` header file. `libsmoldyn` comes with one header file, libsmoldyn.h,
which has function declarations for all of the `libsmoldyn` functions. For most
`libsmoldyn` applications, this is the only header file that you will need to
include. For Mac and Linux, it is typically installed to /usr/local/include.
This is one of the standard system paths, so include it with

```c
#include "libsmoldyn.h"
```

If the `libsmoldyn.h` header file is in some other directory or if your system
isn’t seeing its path as a system path, then include the file using double
quotes rather than angle brackets and/or include more information about the
path. For example, 

```c
#include "libsmoldyn.h"
```

`libsmoldyn.h` calls a second header file, `smoldyn.h`, which is also typically
installed to `/usr/local/include/`. If you plan to access the Smoldyn data
structure directly, then you will also need to include it with `#include
<smoldyn.h>`. In general, it is safe to read from this data structure but it
can be dangerous to write to it unless you really know what you are doing.
Also, working with this data structure directly bypasses one of the benefits of
using `libsmoldyn`, which is that the interface should be relatively immune to
future Smoldyn developments, because different aspects of the internal data
structure get changed once in a while.

The `smoldyn.h` header calls yet another header file, `smoldynconfigure.h`,
which is also installed by default in `/usr/local/include/`. That file is
automatically generated by the build system. It describes what Smoldyn
components are included in the build, what system the build was compiled for,
etc. This might be helpful to include for some applications.

### Compiling example

In the `examples/S97_libsmoldyn/testcode/` directory, you’ll find the
testcode.c program. To compile this source code to object code, enter:

```bash
$ gcc -Wall -O0 -g -c testcode.c
```

The compile flags `-O0 -g` aren’t necessary but can be useful for
debugging purposes. If compiling doesn’t work at this stage, it’s
probably because you’re missing the header files. Make sure that you
have libsmoldyn.h, smoldyn.h, and smoldyn_config.h in the
`/usr/local/include` directory.

### Linking

Linking the different object files together to create an executable that
actually runs is often one of the greatest frustrations of using
software libraries. It should be easy but usually isn’t.

The `libsmoldyn` library can be linked statically, meaning that the
`libsmoldyn` code will be copied into the final result, or it can be
linked dynamically, so that the final result will simply reference the
`libsmoldyn` code that is stored separately. Dynamic linking is somewhat
more elegant in that it doesn’t create unnecessary copies of the
compiled code. It can also be easier. On the other hand, it’s less
convenient if you plan to distribute your software, because then you
need to make sure that you distribute the `libsmoldyn` header file and
library code along with your own software. Also, I can only get the gdb
debugger to help find errors within `libsmoldyn` if the library is
statically linked.

The `libsmoldyn` static library is called libsmoldyn_static.a and the
`libsmoldyn` dynamic library is called libsmoldyn_shared.so (on Linux;
the .so suffix is replaced by .dylib on a Mac and by .dll on Windows).
By default, these libraries are installed to /usr/local/lib/.

#### Linking examples

Following are several example for static and dynamic linking. They are
shown for C; if you used C++, then link with g++ instead of gcc. The
linking options for Smoldyn compiled with OpenGL are shown for
Macintosh; these lines are simpler for other systems.

I have had a hard time getting static linking working on a Mac, although
apparently it works fine on Ubuntu. The problem is that it doesn’t find
the standard C++ library. The solution is to build the Smoldyn library
without NSV, so that the standard C++ library isn’t needed. I also
commented out a few `throw` statements from smolsim.c and libsmoldyn.c
for this purpose.

Static link, no OpenGL:

```bash
$ gcc testcode.o /usr/local/lib/libsmoldyn_static.a -o testcode
```

Static link, with OpenGL:

```bash
$ gcc testcode.o /usr/local/lib/libsmoldyn_static.a  \
    -I/System/Library/Frameworks/OpenGL.framework/Headers \
    -I/System/Library/Frameworks/GLUT.framework/Headers -framework GLUT \
    -framework OpenGL -framework Cocoa \
     -L/System/Library/Frameworks/OpenGL.framework/Libraries -o testcode \
    -ltiff
```

Dynamic link, no OpenGL:

```bash
$ gcc testcode.o -o testcode -lsmoldyn_shared`
```

Dynamic link, with OpenGL:

```bash
$ gcc test1.o -L/usr/local/lib \
     -I/System/Library/Frameworks/OpenGL.framework/Headers \
     -I/System/Library/Frameworks/GLUT.framework/Headers -framework GLUT \
     -framework OpenGL -framework Cocoa \
     -L/System/Library/Frameworks/OpenGL.framework/Libraries -o test1 \
     -lsmoldyn_shared -ltiff
```

### Using smaller versions of `libsmoldyn`

As a default, Smoldyn and `libsmoldyn` are compiled with all of their
components. However, they can also be compiled without OpenGL, without hybrid
simulation (NSV) support, without LibTiff support, etc. Removing these
components removes some aspects of the functionality, obviously, but can also
simplify linking.

Following is a simple diagram for Smoldyn’s code dependencies. Each file
depends on the files that are indented below it.

- Smoldyn  
- OpenGL  
- libTiff  
- zlib  
- libiconv  
- NSV  
- boost  
- VTK  

To build with fewer components, you will need to run CMake to compile
Smoldyn. This is described in more detail in the Smoldyn Code
Documentation, but summarized here for convenience.

I prefer to run CMake from a command line interface. At a command line
interface, change directories to cmake. Every time you change CMake settings,
you’ll probably want to do a clean build. To do so, enter  `rm -r *`, while in
the cmake directory (verify that you're in this directory!), to remove any
prior build results. If you’re asked about whether m`anifest.txt` should be
removed, say yes; this file shows the directories where Smoldyn was installed
previously, thus providing information for you to remove it. For a default
build, enter `cmake ..`  A few test results will be printed out, and then
configuring will be complete. When CMake is done, it will have written a lot of
stuff to the cmake directory. Important files are `Makefile`, which is the
standard Makefile for the code and also `smoldynconfigure.h`, which is a C
header file that the Smoldyn code uses for knowing what some important build
parameters are.

Once configuring is complete, enter `make`. Hopefully, Smoldyn will
build, again with build files being put into the cmake directory.
Finally, enter `sudo make install` and enter your password, to install
Smoldyn to the usual place (`/usr/local/bin` on Linux and Mac systems).

For custom builds, you need to set various options to non-default settings.
With a command line interface, list each non-default option on the command line
after the `cmake ..` start. Following are some helpful build options:

| Smoldyn option                  | default   | effect when ON                           |
| :------------------------------ | :-------- | :--------------------------------------- |
| `-DOPTION_VCELL`                | `OFF`     | Build for inclusion within VCell         |
| `-DOPTION_NSV`                  | `ON`      | Build with Next Subvolume support        |
| `-DOPTION_PDE`                  | `OFF`     | Build with support for PDE simulation    |
| `-DOPTION_VTK`                  | `OFF`     | Build sith support for VTK visualization |
| `-DOPTION_STATIC`               | `OFF`     | Build using static libraries             |
| `-DOPTION_USE_OPENGL`           | `ON`      | Build with graphics support              |
| `-DOPTION_USE_LIBTIFF`          | `ON`      | Build with LibTiff support               |
| `-DOPTION_TARGET_SMOLDYN`       | `ON`      | Build stand-alone Smoldyn program        |
| `-DOPTION_TARGET_LIBSMOLDYN`    | `OFF`     | Build `libsmoldyn` library                 |

| CMake option                    | default   | function                                 |
|:--------------------------------|:----------|:-----------------------------------------|
| `-DCMAKE_BUILD_TYPE`            | `Release` | Choose CMake build type                  |
| `-DCMAKE_CXX_COMPILER:FILEPATH` | clang     | Compile with specific compiler           |

For example, the following line builds Smoldyn and `libsmoldyn` for
debugging and without the hybrid simulation support:


```bash
$ cmake .. -DCMAKE_BUILD_TYPE=Debug -DOPTION_TARGET_LIBSMOLDYN=ON -DOPTION_NSV=OFF
```

### Error trapping

Every function in `libsmoldyn` checks that its input values are acceptable and
also that no errors arise in the function execution. These errors are returned
to the host library in a number of ways. Most `libsmoldyn` functions (e.g.
`smolRunSim`) return any error codes directly, which makes it easy to see if an
error arose. However, a few functions (e.g.  `smolNewSim`) return other types
of values and so return some other indication of success or failure (e.g.
`NULL`). In addition, some functions can raise warnings, which indicate that
behavior is unusual but not incorrect.

For all of these errors and warnings, get the details of the problem using the
function `smolGetError`, which will return the error code, the name of the
function where the error arose, and a descriptive error string. This will also
clear the error, if desired. If errors are not cleared, they are left until
they are overwritten by subsequent errors.  Warnings are also left until they
are cleared or overwritten.

When writing code, it can be helpful to put `libsmoldyn` into its debugging mode
using the `smolSetDebugMode` function. Doing this causes any errors that arise
to be displayed to stderr.

The possible error codes are declared in libsmoldyn.h with:

```c
enum ErrorCode {ECok=0,ECnotify=-1,ECwarning=-2,ECnonexist=-3,ECall=-4,ECmissing=-5,ECbounds=-6,ECsyntax=-7,ECerror=-8,ECmemory=-9,ECbug=-10,ECsame=-11};
```

Their interpretations are:

| value | code         | interpretation                                          |
| :---: | :----------- | :------------------------------------------------------ |
|   0   | `ECok`       | no error                                                |
|  -1  | `ECnotify`   | message about correct behavior                          |
|  -2  | `ECwarning`  | unusual but not incorrect behavior                      |
|  -3  | `ECnonexist` | a function input specifies an item that doesn’t exist   |
|  -4  | `ECsame`     | error code should be unchanged from a prior code        |
|  -5  | `ECall`      | an argument of `all` was found and may not be permitted |
|  -6  | `ECmissing`  | a necessary function input parameter is missing         |
|  -7  | `ECbounds`   | a function input parameter is out of bounds             |
|  -8  | `ECsyntax`   | function inputs don’t make syntactical sense            |
|  -9  | `ECerror`    | unspecified error condition                             |
| -10  | `ECmemory`   | Smoldyn was unable to allocate the necessary memory     |
| -11  | `ECbug`      | error arose which should not have been possible         |

### Error checking system internal to libsmoldyn.c

This section describes how to write `libsmoldyn` functions using error
checking. While it is an essential part of all `libsmoldyn` functions, these
details are not important for most `libsmoldyn` users.

1.  The first line of every `libsmoldyn` function should be `const char
    *funcname="`*function_name*``;`. This name will be returned with
    any error message to tell the user where the error arose.

2.  Within the function, check for warnings or errors with the `LCHECK`
    macro. The macro format is
    `LCHECK(`*condition*`,funcname,`*error_code*`,"`*message*``);`.
    This checks that the test *condition* is true, and issues a
    notification, warning, or error when this is not the case. The
    *message* should be a descriptive message that is under 256
    characters in length.

3.  Most functions return an `enum ErrorCode`. If this is the case for
    your function, and your function might return a notification and/or
    a warning, then end the main body of the function with `return
    libwarncode;`. If it cannot return a notification or a warning, then
    end it with `return ECok;`. Finally, if it does not return an ``enum
    ErrorCode`, then it needs to return some other error condition that
    will tell the user to check for errors using `smolGetError`.

4.  After the main body of the function, add a goto target called
    `failure:`.

5.  Assuming the function returns an ``enum ErrorCode`", end the
    function with `return liberrorcode;`.

The `smolSetTimeStep` function provides an excellent and simple example
of how `libsmoldyn` functions typically address errors. It is:

```c
enum ErrorCode smolSetTimeStep(simptr sim,double timestep) {
    const char *funcname="smolSetTimeStep";

    LCHECK(sim,funcname,ECmissing,"missing sim");
    LCHECK(timestep>0,funcname,ECbounds,"timestep is not > 0");
    simsettime(sim,timestep,3);
    return ECok;
 failure:
    return liberrorcode; }
```

The `smolGet...Index` functions are worth a comment. Each of these functions
returns the index of an item, such as a species or a surface, based on the name
of the item. If the name is not found or other errors arise, then these
functions return the error code, cast as an integer.  Also, if the name is
`all`, then these functions return the error code `ECall` and set the error
string `species cannot be ‘all’", or equivalent. A typical use of these
functions is seen in `smolSetSpeciesMobility`, which includes the following
code:

```c
i=smolGetSpeciesIndex(sim,species);
if(i==(int)ECall) smolClearError();
else LCHECK(i>0,funcname,ECsame,NULL);
```

In this particular case, this function permits an input of `all`, so it clears
errors that arise from this return value, and leaves `i` as a negative value
for later use.

## `libsmoldyn` quick function guide

The `libsmoldyn` functions correspond relatively closely to the Smoldyn
language statements, although not perfectly. However, all functionality
should be available using either method. The following table lists the
correspondences. Statements preceded by asterisks need to be either
entered in statement blocks or preceded by the statement’s context (e.g.
with `surface` *name*). Where correspondence does not apply, the table
lists `N/A`.

| Statement                          | `libsmoldyn` function                                  |
| :--------------------------------- | :--------------------------------------------------- |
| `#`                                | N/A                                                  |
| `/* ... */`                        | N/A                                                  |
| read_file                         | `smolLoadSimFromFile`, `smolReadConfigString`        |
| end_file                          | N/A                                                  |
| define                             | N/A                                                  |
| define_global                     | N/A                                                  |
| undefine                           | N/A                                                  |
| ifdefine                           | N/A                                                  |
| ifundefine                         | N/A                                                  |
| else                               | N/A                                                  |
| endif                              | N/A                                                  |
| display_define                    | N/A                                                  |
| N/A                                | `smolSetError`                                       |
| N/A                                | `smolGetError`                                       |
| N/A                                | `smolClearError`                                     |
| N/A                                | `smolSetDebugMode`                                   |
| N/A                                | `smolErrorCodeToString`                              |
|                                    |                                                      |
| dim                                | `smolNewSim`                                         |
| boundaries                         | `smolNewSim`, `smolSetBoundaryType`                  |
| low_wall                          | `smolNewSim`, `smolSetBoundaryType`                  |
| high_wall                         | `smolNewSim`, `smolSetBoundaryType`                  |
| time_start                        | `smolSetSimTimes`, `smolSetTimeStart`                |
| time_stop                         | `smolSetSimTimes`, `smolSetTimeStop`                 |
| time_step                         | `smolSetSimTimes`, `smolSetTimeStep`                 |
| time_now                          | `smolSetTimeNow`                                     |
|                                    |                                                      |
| species                            | `smolAddSpecies`                                     |
| N/A                                | `smolGetSpeciesIndex`                                |
| N/A                                | `smolGetSpeciesName`                                 |
| difc                               | `smolSetSpeciesMobility`                             |
| difm                               | `smolSetSpeciesMobility`                             |
| drift                              | `smolSetSpeciesMobility`                             |
| mol                                | `smolAddSolutionMolecules`                           |
| surface_mol                       | `smolAddSurfaceMolecules`                            |
| compartment_mol                   | `smolAddCompartmentMolecules`                        |
| molecule_lists                    | `smolAddMolList`                                     |
| mol_list                          | `smolAddSpecies`, `smolSetMolList`                   |
| N/A                                | `smolGetMolListIndex`                                |
| N/A                                | `smolGetMolListName`                                 |
| max_mol                           | `smolSetMaxMolecules`                                |
| N/A                                | `smolGetMoleculeCount`                               |
|                                    |                                                      |
| graphics                           | `smolSetGraphicsParams`                              |
| graphic_iter                      | `smolSetGraphicsParams`                              |
| graphic_delay                     | `smolSetGraphicsParams`                              |
| frame_thickness                   | `smolSetFrameStyle`                                  |
| frame_color                       | `smolSetFrameStyle`                                  |
| grid_thickness                    | `smolSetGridStyle`                                   |
| grid_color                        | `smolSetGridStyle`                                   |
| background_color                  | `smolSetBackgroundStyle`                             |
| display_size                      | `smolSetMoleculeStyle`                               |
| color                              | `smolSetMoleculeStyle`                               |
| tiff_iter                         | `smolSetTiffParams`                                  |
| tiff_name                         | `smolSetTiffParams`                                  |
| tiff_min                          | `smolSetTiffParams`                                  |
| tiff_max                          | `smolSetTiffParams`                                  |
| light                              | `smolSetLightParams`                                 |
| text_color                        | `smolSetTextStyle`                                   |
| text_display                      | `smolAddTextDisplay`                                 |
|                                    |                                                      |
| output_root                       | `smolSetOutputPath`                                  |
| output_files                      | `smolAddOutputFile`                                  |
| append_files                      | `smolAddOutputFile`                                  |
| output_file_number               | `smolAddOutputFile`                                  |
| cmd                                | `smolAddCommand`, `smolAddCommandFromString`         |
|                                    |                                                      |
| start_surface                     | `smolAddSurface`                                     |
| new_surface                       | `smolAddSurface`                                     |
| <span>\*</span> name               | `smolAddSurface`                                     |
| N/A                                | `smolGetSurfaceIndex`                                |
| N/A                                | `smolGetSurfaceName`                                 |
| action                             | `smolSetSurfaceAction`                               |
| rate                               | `smolSetSurfaceRate`                                 |
| rate_internal                     | `smolSetSurfaceRate`                                 |
| color                              | `smolSetSurfaceFaceStyle`, `smolSetSurfaceEdgeStyle` |
| thickness                          | `smolSetSurfaceEdgeStyle`                            |
| stipple                            | `smolSetSurfaceEdgeStyle`                            |
| polygon                            | `smolSetSurfaceFaceStyle`                            |
| shininess                          | `smolSetSurfaceFaceStyle`                            |
| panel                              | `smolAddPanel`                                       |
| N/A                                | `smolGetPanelIndex`                                  |
| N/A                                | `smolGetPanelName`                                   |
| jump                               | `smolSetPanelJump`                                   |
| neighbors                          | `smolAddPanelNeighbor`                               |
| unbounded_emitter                  | `smolAddSurfaceUnboundedEmitter`                     |
| end_surface                        | N/A                                                  |
| epsilon                            | `smolSetSurfaceSimParams`                            |
| margin                             | `smolSetSurfaceSimParams`                            |
| neighbor_dist                     | `smolSetSurfaceSimParams`                            |
|                                    |                                                      |
| start_compartment                 | `smolAddCompartment`                                 |
| new_compartment                   | `smolAddCompartment`                                 |
| <span>\*</span> name               | `smolAddCompartment`                                 |
| N/A                                | `smolGetCompartmentIndex`                            |
| N/A                                | `smolGetCompartmentName`                             |
|  surface                            | `smolAddCompartmentSurface`                          |
|  point                              | `smolAddCompartmentPoint`                            |
|  compartment                        | `smolAddCompartmentLogic`                            |
|  end_compartment                   | N/A                                                  |
|                                    |                                                      |
| reaction                           | `smolAddReaction`                                    |
| N/A                                | `smolGetReactionIndex`                               |
| N/A                                | `smolGetReactionName`                                |
| reaction_cmpt                     | `smolSetReactionRegion`                              |
| reaction_surface                  | `smolSetReactionRegion`                              |
| reaction_rate                     | `smolAddReaction`, `smolSetReactionRate`             |
| confspread_radius                 | `smolSetReactionRate`                                |
| binding_radius                    | `smolSetReactionRate`                                |
| reaction_probability              | `smolSetReactionRate`                                |
| reaction_production               | `smolSetReactionRate`                                |
| reaction_permit                   | not supported                                        |
| reaction_forbid                   | not supported                                        |
| product_placement                 | `smolSetReactionProducts`                            |
|                                    |                                                      |
| start_port                        | `smolAddPort`                                        |
| new_port                          | `smolAddPort`                                        |
| <span>\*</span> name               | `smolAddPort`                                        |
| N/A                                | `smolGetPortIndex`                                   |
| N/A                                | `smolGetPortName`                                    |
| surface                           | `smolAddPort`                                        |
| face                                | `smolAddPort`                                        |
| end_port                            | N/A                                                  |
| N/A                                | `smolAddPortMolecules`                               |
| N/A                                | `smolGetPortMolecules`                               |
|                                    |                                                      |
| rand_seed                         | `smolSetRandomSeed`                                  |
| accuracy                           | not supported                                        |
| molperbox                          | `smolSetPartitions`                                  |
| boxsize                            | `smolSetPartitions`                                  |
| gauss_table_size                 | not supported                                        |
| epsilon                            | `smolSetSurfaceSimParams`                            |
| margin                             | `smolSetSurfaceSimParams`                            |
| neighbor_dist                     | `smolSetSurfaceSimParams`                            |
| pthreads                           | not supported                                        |
|                                    |                                                      |
| N/A                                | `smolUpdateSim`                                      |
| N/A                                | `smolRunTimeStep`                                    |
| N/A                                | `smolRunSim`                                         |
| N/A                                | `smolRunSimUntil`                                    |
| N/A                                | `smolFreeSim`                                        |
| N/A                                | `smolDisplaySim`                                     |
| N/A                                | `smolPrepareSimFromFile`                             |


# `libsmoldyn` header file

Following is the entire `libsmoldyn` header file, `libsmoldyn.h`. This lists
all of the function declarations. If there is a discrepancy between
declarations listed here and those listed in following sections, the ones shown
here are almost certainly the correct ones. This file references smoldyn.h,
which lists all of the data structure declarations and enumerated type
definitions.

If you compiled and installed Smoldyn using the default configuration, both
files should be in your `/usr/local/include/smoldyn` directory. Also in this
directory is the `smoldyn_config.h` file. This file was used for compiling
Smoldyn and `libsmoldyn` but is not needed afterwards.  Nevertheless, it’s
copied to the /usr/local/include/smoldyn directory so that programs that call
`libsmoldyn` can know what options `libsmoldyn` was built with.


# `libsmoldyn` functions

## General comments

None of the functions allocate memory, except within the simulation data
structure. This means, for example, that all functions that return
strings do not allocate these strings themselves, but instead write the
string text to memory that the library user allocated and gave to the
function. All strings are fixed at `STRCHAR` characters, where this
constant is defined in `string2.h` to 256 characters.

## Miscellaneous

- `double smolGetVersion(void);` Returns the Smoldyn version number.

## Errors

  - `void smolSetError(const char *errorfunction,enum ErrorCode errorcode,const char *errorstring);`  
    This function is probably not useful for most users. Sets the
    `libsmoldyn` error code to `errorcode`, error function to
    `errorfunction`, and error string to `errorstring`. The sole
    exception is if `errorcode` is `ECsame` then this does nothing and
    simply returns. Back to it’s normal operation, this also either sets
    or clears the `libsmoldyn` warning code, as appropriate. If
    `errorstring` is entered as `NULL`, this clears the current error
    string, and similarly for `errorfunction`.

  - `enum ErrorCode smolGetError(char *errorfunction,char *errorstring,int clearerror);`  
    Returns the current `libsmoldyn` error code directly, returns the
    function where the error occurred in `errorfunction` if it is not
    `NULL`, and returns the error string in `errorstring` if it is not
    `NULL`. Set `clearerror` to 1 to clear the error and 0 to leave any
    error condition unchanged.

  - `void smolClearError(void);`  
      
    Clears any error condition.

  - `void smolSetDebugMode(int debugmode);`  
      
    Enter `debugmode` as 1 to enable debugging and 0 to disable
    debugging. When debug mode is turned on, all errors are displayed to
    stderr, as are all cleared errors. By turning on debug mode, you can
    often avoid checking for errors with additional code and you also
    typically don’t need to call `smolGetError`.

  - `char* smolErrorCodeTostring(enum ErrorCode erc,char *string);`  
      
    Returns a string both directly and in `string` that corresponds to
    the error code in `erc`. For example, if `erc` is `ECmemory`, this
    returns the string `memory`.

## Sim structure

  - `simptr smolNewSim(int dim,double *lowbounds,double *highbounds);`  
    Creates and returns a new sim structure. The structure is
    initialized for a `dim` dimensional system that has boundaries
    defined by the points `lowbounds` and `highbounds`. Boundaries are
    transmitting (modify them with `smolSetBoundaryType`). Returns
    `NULL` upon failure.

  - `enum ErrorCode smolUpdateSim(simptr sim);`  
    Updates the simulation structure. This calculates all simulation
    parameters from physical parameters, sorts lists, and generally does
    everything required to make a simulation ready to run. It may be
    called multiple times.

  - `enum ErrorCode smolRunTimeStep(simptr sim);`  
    Runs one time step of the simulation. Returns an error if the
    simulation terminates unexpectedly during this time step or a
    warning if it terminates normally.

  - `enum ErrorCode smolRunSim(simptr sim);`  
    Runs the simulation until it terminates. Returns an error if the
    simulation terminates unexpectedly during this time step or a
    warning if it terminates normally.

  - `enum ErrorCode smolRunSimUntil(simptr sim,double breaktime);`  
    Runs the simulation either until it terminates or until the
    simulation time equals or exceeds `breaktime`.

  - `enum ErrorCode smolFreeSim(simptr sim);`  
    Frees the simulation data structure.

  - `enum ErrorCode smolDisplaySim(simptr sim);`  
    Displays all relevant information about the simulation system to
    stdout.

## Read configuration file

  - `simptr smolPrepareSimFromFile(char *filepath,char *filename,char *flags);`  
    Reads the Smoldyn configuration file that is at `filepath` and has
    file name `filename`, sets it up, and outputs simulation diagnostics
    to stdout. Returns the sim structure, or `NULL` if an error
    occurred. `flags` are the command line flags that are entered for
    normal Smoldyn use. Either or both of `filepath` and `flags` can be
    sent in as `NULL` if there is nothing to report. After this function
    runs successfully, it should be possible to call `smolRunSim` or
    `smolRunTimeStep`.

  - `enum ErrorCode smolLoadSimFromFile(char *filepath,char *filename,simptr *simpointer,char *flags);`  
    Loads part or all of a sim structure from the file that is at
    `filepath` and has file name `filename`. Send in `simpointer` as a
    pointer to sim, where sim may be an existing simulation structure
    that this function will append or `NULL` if it is to be created by
    this function. `flags` are the command line flags that are entered
    for normal Smoldyn use. Either or both of `filepath` and `flags` can
    be sent in as `NULL` if there is nothing to report. After this
    function runs successfully, call `smolUpdateSim` to calculate
    simulation parameters.

  - `enum ErrorCode smolReadConfigString(simptr sim,char *statement,char *parameters);`  
    Reads and processes what would normally be a single line of a
    configuration file. The first word of the line is the statement
    name, entered here as `statement`, while the rest of the line is
    entered as `parameters`. Separate different parameters with spaces.
    The same parser is used as for normal Smoldyn configuration files.
    This function does not make use of block style input formatting,
    such as for surface definitions. This means that a new surface needs
    to declared with `new_surface` **name** and all subsequent surface
    definitions need to start with surface **name**. Analogous rules
    apply to compartments and port.

## Simulation settings

  - `enum ErrorCode smolSetSimTimes(simptr sim,double timestart,double timestop,double timestep);`  
    Sets all of the simulation time parameters to the values entered
    here. In addition the simulation time `now` is set to `timestart`.

  - `enum ErrorCode smolSetTimeStart(simptr sim,double timestart);`  
    Sets the simulation starting time.

  - `enum ErrorCode smolSetTimeStop(simptr sim,double timestop);`  
    Sets the simulation stopping time.

  - `enum ErrorCode smolSetTimeNow(simptr sim,double timenow);`
    Sets the simulation current time.

  - `enum ErrorCode smolSetTimeStep(simptr sim,double timestep);`
    Sets the simulation time step, which must be greater than 0.

  - `enum ErrorCode smolSetRandomSeed(simptr sim,double seed);`
    Sets the random number generator seed to `seed` if `seed` is at
    least 0, and sets it to the current time value if `seed` is less
    than 0.

  - `enum ErrorCode smolSetPartitions(simptr sim,char *method,double value);`
    Sets the virtual partitions in the simulation volume. Enter `method`
    as `molperbox` and then enter `value` with the requested number of
    molecules per partition volume; the default, which is used if this
    function is not called at all, is a target of 4 molecules per box.
    Or, enter `method` as `boxsize` and enter `value` with the requested
    partition spacing. In this latter case, the actual partition spacing
    may be larger or smaller than the requested value in order to fit an
    integer number of partitions into each coordinate of the simulation
    volume.

## Graphics

  - `enum ErrorCode smolSetGraphicsParams(simptr sim,char *method,int timesteps,double delay);`  
    Sets basic simulation graphics parameters. Enter `method` as `none`
    for no graphics (the default), `opengl` for fast but minimal OpenGL
    graphics, `opengl_good` for improved OpenGL graphics,
    `opengl_better` for fairly good OpenGL graphics, or as `NULL` to
    not set this parameter currently. Enter `timesteps` with a positive
    integer to set the number of simulation time steps between graphics
    renderings (1 is the default) or with a negative number to not set
    this parameter currently. Enter `delay` as a non-negative number to
    set the minimum number of milliseconds that must elapse between
    subsequent graphics renderings in order to improve visualization (0
    is the default) or as a negative number to not set this parameter
    currently.

  - `enum ErrorCode smolSetTiffParams(simptr sim,int timesteps,char *tiffname,int lowcount,int highcount);`  
    Sets parameters for the automatic collection of TIFF format
    snapshots of the graphics window. `timesteps` is the number of
    simulation timesteps that should elapse between subsequent
    snapshots, `tiffname` is the root filename of the output TIFF files,
    `lowcount` is a number that is appended to the filename of the first
    snapshot and which is then incremented for subsequent snapshots, and
    `highcount` is the last numbered file that will be collected. Enter
    negative numbers for `timesteps`, `lowcount`, and/or `highcount` to
    not set these parameters, and enter `NULL` for `tiffname` to not set
    the file name.

  - `enum ErrorCode smolSetLightParams(simptr sim,int lightindex,double *ambient,double *diffuse,double *specular,double *position);`  
    Sets the lighting parameters that are used for the rendering method
    `opengl_better`. Enter `lightindex` as -1 for the global ambient
    light (in which case `diffuse`, `specular`, and `position` should
    all be `NULL`) or as 0 to 8 for one of the 8 light sources. For each
    light source, you can specify the 4-value color vector for the
    light’s ambient, diffuse, and specular properties (all values
    should be between 0 and 1). You can also specify the 3-dimensional
    position for the light. To not set a property, just enter the
    respective vector as `NULL`.

  - `enum ErrorCode smolSetBackgroundStyle(simptr sim,double *color);`  
    Sets the color of the graphics display background. `color` is a
    4-value vector with red, green, blue, and alpha values.

  - `enum ErrorCode smolSetFrameStyle(simptr sim,double thickness,double *color);`  
    Sets the thickness and the color of the wire frame that outlines the
    simulation system in the graphics window. Enter `thickness` as 0 for
    no frame, as a positive number for the number of points in
    thickness, or as a negative number to not set this parameter. Enter
    `color` as a 4-value vector with the frame color, or as `NULL` to
    not set it.

  - `enum ErrorCode smolSetGridStyle(simptr sim,double thickness,double *color);`  
    Sets the thickness and the color of a grid that shows where the
    partitions are that separate Smoldyn’s virtual boxes. Enter
    `thickness` as 0 for no grid, as a positive number for the number of
    points in thickness, or as a negative number to not set this
    parameter. Enter `color` as a 4-value vector with the grid color, or
    as `NULL` to not set it.

  - `enum ErrorCode  smolSetTextStyle(simptr sim,double *color);`  
    Sets the color of any text that is displayed to the graphics window.
    `color` is a 4-value vector with red, green, blue, and alpha values.

  - `enum ErrorCode smolAddTextDisplay(simptr sim,char *item);`  
    Adds `item` to the list of things that Smoldyn should display as
    text to the graphics window. Currently supported options are `time`
    and the names of species and, optionally, their states. For species
    and states, the graphics window shows the number of molecules.

## Runtime commands

  - `enum ErrorCode smolSetOutputPath(simptr sim,char *path);`  
    Sets the file path for text output files to `path`.

  - `enum ErrorCode  smolAddOutputFile(simptr sim,char *filename,int suffix,int append);`  
    Declares the file called `filename` as a file for output by one or
    more runtime commands. Note that spaces are not permitted in the
    file name. If `suffix` is non-negative, then the file name is
    suffixed by this integer, which can be helpful for creating output
    file stacks. Enter `append` as 1 if any current file should simply
    be appended, or to 0 if any current file should be overwritten.

  - `enum ErrorCode smolAddCommand(simptr sim,char type,double on,double off,double step,double multiplier,char *commandstring);`  
    Adds a run-time command to the simulation, including its timing
    instructions. This function should generally be called after
    `smolSetSimTimes` to make sure that command times get set correctly.
    The following table lists the command type options along with the
    other parameters that are used for each type. Parameters that are
    not required are simply ignored. The `commandstring` is the command
    name followed by any command parameters.
    
    | `type` | meaning                   |   `on`    |   `off`   |     `step`     | `multiplier` |
    | :----- | :------------------------ | :-------: | :-------: | :------------: | :----------: |
    |        |                           |           |           |                |              |
    | `b`    | before simulation         |    \-     |    \-     |       \-       |      \-      |
    | `a`    | after simulation          |    \-     |    \-     |       \-       |      \-      |
    | `@`    | at fixed time             |   time    |    \-     |       \-       |      \-      |
    | `i`    | fixed intervals           |  time on  | time off  |   time step    |      \-      |
    | `x`    | exponential intervals     |  time on  | time off  | min. time step |  multiplier  |
    |        |                           |           |           |                |              |
    | `B`    | before simulation         |    \-     |    \-     |       \-       |      \-      |
    | `A`    | after simulation          |    \-     |    \-     |       \-       |      \-      |
    | `&`    | at fixed iteration        | iteration |    \-     |       \-       |      \-      |
    | `I`    | fixed iteration intervals | iter. on  | iter. off |   iter. step   |      \-      |
    | `E`    | every time step           |    \-     |    \-     |       \-       |      \-      |
    | `N`    | every n’th time step      |    \-     |    \-     |   iter. step   |      \-      |
    

  - `enum ErrorCode`  
    `smolAddCommandFromString(simptr sim,char *string);`  
    Defines a runtime command, including its execution timing
    parameters, from the string `string`. This string should be
    identical to ones used in configuration files, except that they do
    not include the `cmd` statement.

## Molecules

  - `enum ErrorCode smolAddSpecies(simptr sim,char *species,char *mollist);`  
    Adds a molecular species named `species` to the system. If you have
    already created species lists and want all states of this species to
    live in a specific list, then enter it in `mollist`; otherwise,
    enter `mollist` as `NULL` or an empty string to request default
    behavior.

  - `int smolGetSpeciesIndex(simptr sim,char *species);`  
    Returns the species index that corresponds to the species named
    `species`. Upon failure, this function returns an error code cast as
    an integer.

  - `char* smolGetSpeciesName(simptr sim,int speciesindex,char *species);`  
    Returns the species name that corresponds to the species index in
    `speciesindex`. The name is returned both in `species` and directly,
    where the latter simplifies function use. Upon failure, this
    function returns `NULL`.

  - `enum ErrorCode smolSetSpeciesMobility(simptr sim,char *species,enum MolecState
    state,double difc,double *drift,double *difmatrix);`  
    Sets any or all of the mobility coefficients for species `species`
    (which may be `all`) and state `state` (which may be `MSall`).
    `difc` is the isotropic diffusion coefficient, `drift` is the drift
    vector, and `difmatrix` is the square of the anisotropic diffusion
    matrix (see the User’s manual). To not set coefficients, enter a
    negative number in `difc` and/or enter a `NULL` pointer in the other
    inputs, respectively.

  - `int smolAddMolList(simptr sim,char *mollist);`  
    Adds a new molecule list, named `mollist`, to the system.

  - `int smolGetMolListIndex(simptr sim,char *mollist);`  
    Returns the list index that corresponds to the list named `mollist`.

  - `char* smolGetMolListName(simptr sim,int mollistindex,char *mollist);`  
    Returns the molecule list name that corresponds to the molecule list
    with index `mollistindex`. The name is returned both in `mollist`
    and directly. On error, this function `NULL`.

  - `enum ErrorCode smolSetMolList(simptr sim,char *species,enum MolecState state,char
    *mollist);`  
    Sets the molecule list for species `species` (which may be `all`)
    and state `state` (which may be `MSall`) to molecule list `mollist`.

  - `enum ErrorCode smolSetMaxMolecules(simptr sim,int maxmolecules); `  
    Sets the maximum number of molecules that can simultaneously exist
    in a system to `maxmolecules`. At present, this function needs to be
    called for a simulation to run, although it will become optional
    once dynamic molecule memory allocation has been written.

  - `enum ErrorCode  smolAddSolutionMolecules(simptr sim,char *species,int number,double *lowposition,double *highposition);`  
    Adds `number` solution state molecules of species `species` to the
    system. They are randomly distributed within the box that has its
    opposite corners defined by `lowposition` and `highposition`. Any or
    all of these coordinates can equal each other to place the molecules
    along a plane or at a point. Enter `lowposition` and/or
    `highposition` as `NULL` to indicate that the respective corner is
    equal to that corner of the entire system volume.

  - `enum ErrorCode smolAddCompartmentMolecules(simptr sim,char *species,int number,char *compartment);`  
    Adds `number` solution state molecules of species `species` to the
    compartment `compartment`. Molecules are randomly distributed within
    the compartment.

  - `enum ErrorCode smolAddSurfaceMolecules(simptr sim,int speciesindex,enum MolecState state,int number,int surface,enum PanelShape panelshape,int panel,double *position);`  
    Adds `number` molecules of species `species` and state `state` to
    surface(s) in the system. It is permissible for `surface` to be
    `all`, `panelshape` to be PSall, and/or `panel` to be `all`. If
    you want molecules at a specific position, then you need to enter a
    specific surface, panel shape, and panel, and then enter the
    position in `position`.

  - `int smolGetMoleculeCount(simptr sim,char *species,enum MolecState state);`  
    Returns the total number of molecules in the system that have
    species `species` (`all` is permitted) and state `state` (`MSall` is
    permitted). Any error is returned as the error code cast as an
    integer.

  - `enum ErrorCode smolSetMoleculeStyle(simptr sim,const char *species,enum MolecState state,double size,double *color);`  
    Sets the graphical display parameters for molecules of species
    `species` (`all` is permitted) and state `state` (`MSall` is
    permitted). Enter `size` with the drawing size (in pixels if
    graphics method is `opengl` and in simulation system length units
    for better drawing methods) or with a negative number to not set the
    size. Enter `color` with the 3-value color vector or with `NULL` to
    not set the color.

## Surfaces

  - `enum ErrorCode smolSetBoundaryType(simptr sim,int dimension,int highside,char type);`  
    Sets the molecule interaction properties for a system boundary that
    bounds the `dimension` axis. Enter `dimension` as -1 to indicate all
    dimensions. Set `highside` to 0 for the lower boundary, to 1 for the
    upper boundary, and to -1 for both boundaries. The boundary type is
    entered in `type` as ‘r’ for reflecting, ‘p’ for periodic, ‘a’ for
    absorbing, or ‘t’ for transmitting. Note that Smoldyn only observes
    these properties if no surfaces are declared; otherwise all
    boundaries are transmitting regardless of what’s entered here.

  - `int smolAddSurface(simptr sim,char *surface);`  
    Adds a surface called `surface` to the system.

  - `int smolGetSurfaceIndex(simptr sim,char *surface);`  
    Returns the surface index that corresponds to the surface named
    `surface`. The index is non-negative. On failure, this returns an
    error code cast as an integer.

  - `char*  smolGetSurfaceName(simptr sim,int surfaceindex,char *surface);`  
    Returns the surface name for surface number `surfaceindex` both
    directly and in the `surface` string. On failure, this returns
    `NULL`.

  - `enum ErrorCode smolSetSurfaceAction(simptr sim,char *surface,enum PanelFace face,char *species,enum MolecState state,enum SrfAction action);`  
    Sets the action that should happen when a molecule of species
    `species` (may be `all`) and state `state` (may be `MSall`) diffuses
    into face `face` (may be `PFboth`) of surface `surface`. The action
    is set to `action`.

  - `enum ErrorCode smolSetSurfaceRate(simptr sim,char *surface,char *species,enum MolecState state,enum MolecState state1,enum MolecState state2,double rate,char *newspecies,int isinternal);`  
    Sets the surface interaction rate(s) for surface `surface` (may be
    `all`) and species `species` (may be `all`) and state `state`. The
    transition being considered is from `state1` to `state2` (this
    function uses the tri-state format for describing surface
    interactions, shown below). The interaction rate is set to `rate`,
    which is interpreted as a probability value for internal use if
    `isinternal` is 1 and as a physical interaction coefficient if
    `isinternal` is 0. If the molecule ends up interacting with the
    surface, it changes to new species `newspecies`. Enter `newspecies`
    as either `NULL` or an empty string to indicate that molecules
    should not change species upon interactions. The molecule states are
    most easily understood with the following table. If the action
    listed in the table is in italics, then the corresponding
    combination of states is not a permitted input.
    
    |                   |         |          |          |             |
    | :---------------- | :-----: | :------: | :------: | :---------: |
    | interaction class |         |  action  |          |             |
    |                   | `state` | `state1` | `state2` |             |
    |                   |  soln   |   soln   |   soln   |  *reflect*  |
    |                   |    "    |    "     |  bsoln   |  transmit   |
    | collision from    |    "    |    "     |  bound   |   adsorb    |
    | solution state    |    "    |  bsoln   |   soln   |  transmit   |
    |                   |    "    |    "     |  bsoln   |  *reflect*  |
    |                   |    "    |    "     |  bound   |   adsorb    |
    |                   |    "    |  bound   |   soln   |   desorb    |
    | action from       |    "    |    "     |  bsoln   |   desorb    |
    | bound state       |    "    |    "     |  bound   | *no change* |
    |                   |    "    |    "     |  bound’  |    flip     |
    |                   |  bound  |   soln   |   soln   |  *reflect*  |
    |                   |    "    |    "     |  bsoln   |  transmit   |
    |                   |    "    |    "     |  bound   |     hop     |
    | collision from    |    "    |    "     |  bound’  |     hop     |
    | bound state       |    "    |  bsoln   |   soln   |  transmit   |
    |                   |    "    |    "     |  bsoln   |  *reflect*  |
    |                   |    "    |    "     |  bound   |     hop     |
    |                   |    "    |    "     |  bound’  |     hop     |
    |                   |    "    |  bound   |   soln   |   desorb    |
    | action from       |    "    |    "     |  bsoln   |   desorb    |
    | bound state       |    "    |    "     |  bound   | *no change* |
    |                   |    "    |    "     |  bound’  |    flip     |
    | impossible        |    "    |  bound’  |   any    | *nonsense*  |
    

  - `int`  
    `smolAddPanel(simptr sim,char *surface,enum PanelShape
    panelshape,char *panel,char *axisstring,double *params);`  
    Adds or modifies a panel of shape `panelshape` of surface `surface`.
    `axisstring` lists any text parameters for the panel, which in
    practice is only a single word that gives the orientation of a
    rectangle panel (e.g. `+0` or `-y`). `params` lists the numerical
    parameters for the panel location, size, and drawing
    characteristics. These are exactly the same parameters that are
    listed for the `panel` statement in Smoldyn configuration files,
    with the sole exception that the first rectangle `parameter` is
    actually a string that is entered in `axisstring`. `panelname` is an
    optional parameter for naming the panel; if it is included and is
    not an empty string, the panel is named `panelname`. If this panel
    name was already used by a panel of the same shape, then this
    function overwrites that panel’s data with the new data. If the name
    was already used by a panel with a different shape, then this
    creates an error, and if the name was not used before, then a new
    panel is created. To use default panel naming, send in `panelname`
    as either `NULL` or as an empty string. In the latter case,
    `panelname` is returned with the newly assigned default name.

  - `int`  
    `smolGetPanelIndex(simptr sim,char *surface,enum PanelShape
    *panelshapeptr,char *panel);`  
    Returns the panel index for the panel called `panel` on surface
    `surface`. If `panelshapeptr` is not `NULL`, this also returns the
    panel shape in `panelshapeptr`. On failure, this returns the error
    code cast as an integer.

  - `char*`  
    `smolGetPanelName(simptr sim,char *surface,enum PanelShape
    panelshape,int panelindex,char *panel);`  
    Returns the name of the panel that is in surface `surface`, has
    shape `panelshape`, and has index `panelindex`, both directly and in
    the string `panel`. On failure, this returns `NULL`.

  - `enum ErrorCode`  
    `smolSetPanelJump(simptr sim,const char *surface,const char
    *panel1,enum PanelFace face1,const char *panel2,enum PanelFace
    face2,int isbidirectional);`  
    Sets a jumping link between face `face1` of panel `panel1` and face
    `face2` of panel `panel2` of surface `surface`. The link goes from
    `panel1` to `panel2` if `bidirectional` is entered as 0 and goes in
    both directions if `bidirectional` is entered as 1. None of the
    surface, panel, or face entries is allowed to be `all`. This does
    not set the actions of any species to `jump`, which has to be done
    using the `smolSetSurfaceAction` function.

  - `enum ErrorCode`  
    `smolAddSurfaceUnboundedEmitter(simptr sim,const char *surface,enum
    PanelFace face,const char *species,double emitamount,double
    *emitposition);`  
    Adds information about a point molecular source so that face `face`
    of surface `surface` can have its absorption properties calculated
    so that the molecular concentrations will become the same as they
    would be if the surface weren’t there at all. The point molecular
    source emits molecules of species `species`, with a rate of
    `emitamount` and is at location `emitposition`. The emission rate
    does not need to be in absolute units, but only has to be correct
    relative to other unbounded emitters. None of the inputs to this
    function are allowed to be `all`.

  - `enum ErrorCode`  
    `smolSetSurfaceSimParams(simptr sim,const char *parameter,double
    value);`  
    Sets the surface simulation parameter named with `parameter` to
    value `value`. The possible parameters are `epsilon`, `margin`, and
    `neighbordist`. In all cases, the defaults are nearly always good,
    although this function allows them to be modified if desired.
    Epsilon is the maximum distance away from a surface that Smoldyn is
    allowed to place a surface-bound molecule. Margin is the distance
    inside from the edge of a surface panel that Smoldyn will place
    surface-bound molecules that hop onto this panel. Neighbor distance
    is the maximum distance over which surface-bound molecules are
    allowed to hop to transition from one panel to a neighboring panel.

  - `enum ErrorCode`  
    `smolAddPanelNeighbor(simptr sim,const char *surface1,const char
    *panel1,const char *surface2,const char *panel2,int reciprocal);`  
    Adds panel `panel2` of surface `surface2` as a neighbor of panel
    `panel1` or surface `surface1`, meaning that surface-bound molecules
    will be allowed to diffuse from `panel1` to `panel2`. These are not
    allowed to be the same panel. Also, `all` values are not permitted.
    Otherwise, essentially any possible entries are legitimate. If
    surface-bound molecules should also be allowed to diffuse from
    `panel2` to `panel1`, enter `reciprocal` as 1; if not, enter
    `reciprocal` as 0.

  - `enum ErrorCode`  
    `smolSetSurfaceStyle(simptr sim,const char *surface,enum PanelFace
    face,enum DrawMode mode,double thickness,double *color,int
    stipplefactor,int stipplepattern,double shininess);`  
    Sets the graphics output style for face `face` of surface `surface`.
    `mode` is the drawing mode; enter it as `DMnone` to not set this
    parameter and otherwise enter it as `DMno` to not draw the surface,
    `DMvert` for vertices, `DMedge` for edges, or `DMface` for faces.
    The `thickness` parameter gives the point size or line width for
    drawing vertices or edges, or can be entered as a negative number to
    not set this parameter. `color` is the 4-value color vector for the
    surface, or can be entered as `NULL` to not set this parameter.
    `stipplefactor` is the repeat distance for the entire edge stippling
    pattern, or can be entered as a negative number to not set it.
    `stipplepattern` is the edge stippling pattern, which needs to be
    between 0 and 0xFFFF, or can be entered as -1 to not set this
    parameter. And `shininess` is the surface shininess, for use with
    lighting in the `opengl_better` graphics display option, or can be
    entered as -1 to not set this parameter. The parameters `thickness`,
    `stipplefactor`, and `stipplepattern` only apply to edge style
    drawing modes and ignore any input in the `face` entry. The
    `shininess` parameter only applies to the face style drawing modes.

## Compartments

  - `int`  
    `smolAddCompartment(simptr sim,char *compartment);`  
    Adds a compartment called `compartment` to the system.

  - `int`  
    `smolGetCompartmentIndex(simptr sim,char *compartment)`  
    Returns the index of the compartment named `compartment`. On
    failure, this returns an error code cast as an integer.

  - `char*`  
    `smolGetCompartmentName(simptr sim,int compartmentindex,char
    *compartment)`  
    Returns the name of the compartment that has index
    `compartmentindex` both directly and in the string `compartment`.
    Returns `NULL` if an error arises.

  - `enum ErrorCode`  
    `smolAddCompartmentSurface(simptr sim,char *compartment,char
    *surface);`  
    Adds surface `surface` as one of the bounding surfaces of
    compartment `compartment`.

  - `enum ErrorCode`  
    `smolAddCompartmentPoint(simptr sim,char *compartment,double
    *point);`  
    Adds `point` as one of the interior-defining points of compartment
    `compartment`.

  - `enum ErrorCode`  
    `smolAddCompartmentLogic(simptr sim,char *compartment,enum CmptLogic
    logic,char *compartment2);`  
    Modifies the current definition of compartment `compartment` using a
    logical rule specified in `logic` and the definition of
    `compartment2`.

## Reactions

  - `enum ErrorCode`  
    `smolAddReaction(simptr sim,const char *reaction,const char
    *reactant1,enum MolecState rstate1,const char *reactant2,enum
    MolecState rstate2,int nproduct,const char **productspecies,enum
    MolecState *productstates,double rate);`  
    Adds reaction named `reaction` to the system. This reaction can have
    up to two reactants, whose species are listed in `reactant1` and
    `reactant2` and whose states are listed in `rstate1` and `rstate2`.
    If the reaction has fewer than two reactants, set either or both of
    `reactant1` and `reactant2` to either `NULL` or an empty string.
    State the number of reaction products in `nproduct`, list their
    species in `productspecies`, and list their states in
    `productstates`. To set the reaction rate, enter it in `rate`;
    otherwise, enter `rate` as a negative number.

  - `int`  
    `smolGetReactionIndex(simptr sim,int *orderptr,char *reaction);`  
    Returns the index and order for the reaction that is named
    `reaction`. If the order is known, send in `orderptr` pointing to
    this value. If it is not known, send in `orderptr` equal to either
    `NULL` or pointing to a negative number; in this case, it will be
    returned pointing to the reaction order, if the reaction was found.
    On failure, this returns the error code, cast as an integer.

  - `char*`  
    `smolGetReactionName(simptr sim,int order,int reactionindex,char
    *reaction);`  
    Returns the name of the reaction that has reaction order `order` and
    index `reactionindex` in the string `reaction`. Also returns the
    result directly. Returns `NULL` if an error arises.

  - `enum ErrorCode`  
    `smolSetReactionRate(simptr sim,int order,char *reaction,double
    rate,int isinternal);`  
    Set the reaction rate to `rate`. If this value is to be interpreted
    as an internal reaction rate parameter, meaning the production rate
    for zeroth order reactions, the reaction probability for first order
    reactions, or the binding radius for second order reactions, then
    set `isinternal` to 1. Rather than calling this function at all,
    it’s usually easier to use the `rate` parameter of the
    `smolAddReaction` function, although that doesn’t cope with internal
    rate values.

  - `enum ErrorCode`  
    `smolSetReactionRegion(simptr sim,const char *reaction,const char
    *compartment,const char *surface);`  
    Limits the spatial region where a reaction can take place to the
    compartment `compartment` and/or the surface `surface`. To not set
    one of these limits, enter `compartment` and/or `surface` as `NULL`.
    To remove a previously set limit, enter `compartment` and/or
    `surface` as the empty string, `".

  - `enum ErrorCode`  
    `smolSetReactionProducts(simptr sim,const char *reaction,enum
    RevParam method,double parameter,const char *product,double
    *position);`  
    Sets the reaction product parameters for reaction `reaction`. At a
    minimum, the `method` reversible parameter is required. Most of
    these methods require a single parameter, entered in `parameter`. A
    few methods also require a product, in `product` and the relative
    position of this product in `position`.
    
    | `method`       |      `parameter`      |   `product`    |    `position`     |
    | :------------- | :-------------------: | :------------: | :---------------: |
    | `RPnone`       |          \-           |       \-       |        \-         |
    | `RPirrev`      |          \-           |       \-       |        \-         |
    | `RPconfspread` |          \-           |       \-       |        \-         |
    | `RPbounce`     |     \(\sigma_u\)      |       \-       |        \-         |
    | `RPpgem`       |       \(\phi\)        |       \-       |        \-         |
    | `RPpgemmax`    |    \(\phi_{max}\)     |       \-       |        \-         |
    | `RPpgemmaxw`   |    \(\phi_{max}\)     |       \-       |        \-         |
    | `RPratio`      | \(\sigma_u/\sigma_b\) |       \-       |        \-         |
    | `RPunbindrad`  |     \(\sigma_u\)      |       \-       |        \-         |
    | `RPpgem2`      |       \(\phi\)        |       \-       |        \-         |
    | `RPpgemmax2`   |    \(\phi_{max}\)     |       \-       |        \-         |
    | `RPratio2`     | \(\sigma_u/\sigma_b\) |       \-       |        \-         |
    | `RPoffset`     |          \-           | product number | relative position |
    | `RPfixed`      |          \-           | product number | relative position |
    

    If `method` is `RPbounce`, then a negative number for the
    `parameter` indicates default bounce behavior, which is that
    molecules are separated by an amount that is equal to their previous
    overlap.

## Ports

  - `enum ErrorCode`  
    `smolAddPort(simptr sim,const char *port,const char *surface,enum
    PanelFace face);`  
    Adds a port to the simulation. The port will be named `port` and
    will port at the `face` face of surface `surface`.

  - `int`  
    `smolGetPortIndex(simptr sim,const char *port);`  
    Returns the index of the port named `port`.

  - `char*`  
    `smolGetPortName(simptr sim,int portindex,char *port);`  
    Returns the name of the port with index `portindex`, both directly
    and in `port`.

  - `enum ErrorCode`  
    `smolAddPortMolecules(simptr sim,const char *port,int nmolec,const
    char *species,double **positions);`  
    Adds `nmolec` molecules to Smoldyn’s import buffer of port `port`.
    These molecules will all have species `species` and state `MSsoln`.
    Enter `positions` as `NULL` to have the molecules positioned
    randomly over the porting surface and as an `nmolec` length list of
    position vectors to have them located at those specific initial
    positions. These initial positions should be close to the porting
    surface, and on the Smoldyn system side of it.

  - `int`  
    `smolGetPortMolecules(simptr sim,const char *port,const char
    *species,enum MolecState state,int remove);`  
    Returns the number of molecules that are in Smoldyn’s export buffer
    of port `port`. Enter `species` with the species of the molecules
    that should be retrieved, or `all` for all species. Enter `state`
    with the states of the molecules that should be retrieved, or
    `MSall` for all states. Enter `remove` with 1 to remove molecules
    from the export buffer after they are retrieved or with 0 to leave
    them in the buffer. If an error arises, this returns the error code
    cast as an integer.
